@@ -1,11 +1,12 @@
 package Dancer2::Plugin::Auth::Extensible;
 
-our $VERSION = '0.613';
+our $VERSION = '0.614';
 
 use strict;
 use warnings;
 use Carp;
-use Dancer2::Core::Types qw(Bool HashRef Str);
+use Dancer2::Core::Types qw(ArrayRef Bool HashRef Int Str);
+use List::Util qw(first);
 use Module::Runtime qw(use_module);
 use Scalar::Util;
 use Session::Token;
@@ -108,11 +109,40 @@ has permission_denied_page_handler => (
 );
 
 has realms => (
-    is          => 'ro',
-    isa         => HashRef,
-    from_config => sub { {} },
+    is      => 'ro',
+    isa     => ArrayRef,
+    default => sub {
+        my @realms;
+        while ( my ( $name, $realm ) = each %{ $_[0]->config->{realms} } ) {
+            $realm->{priority} ||= 0;
+            push @realms, { name => $name, %$realm };
+        };
+        return [ sort { $b->{priority} <=> $a->{priority} } @realms ];
+    },
 );
 
+has realm_names => (
+    is      => 'lazy',
+    isa     => ArrayRef,
+    default => sub {
+        return [ map { $_->{name} } @{ $_[0]->realms } ];
+    },
+);
+
+has realm_count => (
+    is      => 'lazy',
+    isa     => Int,
+    default => sub { return scalar @{ $_[0]->realms } },
+);
+
+# return realm config hash reference by name
+sub realm {
+    my ( $self, $name ) = @_;
+    croak "realm name not provided" unless $name;
+    my $realm = first { $_->{name} eq $name } @{ $self->realms };
+    return $realm;
+}
+
 has record_lastlogin => (
     is          => 'ro',
     isa         => Bool,
@@ -184,15 +214,14 @@ sub BUILD {
     my $plugin = shift;
     my $app    = $plugin->app;
 
-    my @realms = keys %{ $plugin->realms }
-      or warn
-      "No Auth::Extensible realms configured with which to authenticate user";
+    warn "No Auth::Extensible realms configured with which to authenticate user"
+      unless $plugin->realm_count;
 
     # Force all providers to load whilst we have access to the full dsl.
     # If we try and load later, then if the provider is using other
     # keywords (such as schema) they will not be available from the dsl.
-    for my $realm (@realms) {
-        $plugin->auth_provider($realm);
+    for my $realm ( @{ $plugin->realm_names } ) {
+        $plugin->auth_provider( $realm );
     }
 
     if ( !$plugin->no_default_pages ) {
@@ -281,7 +310,7 @@ sub auth_provider {
 
     # OK, we need to find out what provider this realm uses, and get an instance
     # of that provider, configured with the settings from the realm.
-    my $realm_settings = $plugin->realms->{$realm}
+    my $realm_settings = $plugin->realm($realm)
       or croak "Invalid realm $realm";
 
     my $provider_class = $realm_settings->{provider}
@@ -305,7 +334,7 @@ sub authenticate_user {
     $plugin->execute_plugin_hook( 'before_authenticate_user',
         { username => $username, password => $password, realm => $realm } );
 
-    my @realms_to_check = $realm ? ($realm) : ( keys %{ $plugin->realms } );
+    my @realms_to_check = $realm ? ($realm) : @{ $plugin->realm_names };
 
     for my $realm (@realms_to_check) {
         $plugin->app->log( debug =>
@@ -334,11 +363,10 @@ sub create_user {
     my $plugin  = shift;
     my %options = @_;
 
-    my @all_realms = keys %{ $plugin->realms };
     croak "Realm must be specified when more than one realm configured"
-      if !$options{realm} && @all_realms > 1;
+      if !$options{realm} && $plugin->realm_count > 1;
 
-    my $realm = delete $options{realm} || $all_realms[0];
+    my $realm = delete $options{realm} || $plugin->realm_names->[0];
     my $email_welcome = delete $options{email_welcome};
 
     my $provider = $plugin->auth_provider($realm);
@@ -370,7 +398,7 @@ sub create_user {
 sub get_user_details {
     my ( $plugin, $username, $realm ) = @_;
 
-    my @realms_to_check = $realm ? ($realm) : ( keys %{ $plugin->realms } );
+    my @realms_to_check = $realm ? ($realm) : @{ $plugin->realm_names };
 
     for my $realm (@realms_to_check) {
         $plugin->app->log(
@@ -420,7 +448,7 @@ sub password_reset_send {
     my @realms_to_check =
       $options{realm}
       ? ( $options{realm} )
-      : ( keys %{ $plugin->realms } );
+      : @{ $plugin->realm_names };
 
     my $username = $options{username}
       or croak "username must be passed to password_reset_send";
@@ -459,11 +487,17 @@ sub password_reset_send {
 }
 
 sub require_all_roles {
-    return _build_wrapper( @_, 'all' );
+    my $plugin = shift;
+    croak "Cannot use require_all_roles since roles are disabled by disable_roles setting"
+      if $plugin->disable_roles;
+    return $plugin->_build_wrapper( @_, 'all' );
 }
 
 sub require_any_role {
-    return _build_wrapper( @_, 'any' );
+    my $plugin = shift;
+    croak "Cannot use require_any_role since roles are disabled by disable_roles setting"
+      if $plugin->disable_roles;
+    return $plugin->_build_wrapper( @_, 'any' );
 }
 
 sub require_login {
@@ -493,7 +527,10 @@ sub require_login {
 }
 
 sub require_role {
-    return _build_wrapper( @_, 'single' );
+    my $plugin = shift;
+    croak "Cannot use require_role since roles are disabled by disable_roles setting"
+      if $plugin->disable_roles;
+    return $plugin->_build_wrapper( @_, 'single' );
 }
 
 sub update_current_user {
@@ -513,11 +550,10 @@ sub update_current_user {
 sub update_user {
     my ( $plugin, $username, %update ) = @_;
 
-    my @all_realms = keys %{ $plugin->realms };
     croak "Realm must be specified when more than one realm configured"
-      if !$update{realm} && @all_realms > 1;
+      if !$update{realm} && $plugin->realm_count > 1;
 
-    my $realm    = delete $update{realm} || $all_realms[0];
+    my $realm    = delete $update{realm} || $plugin->realm_name->[0];
     my $provider = $plugin->auth_provider($realm);
     my $updated  = $provider->set_user_details( $username, %update );
     my $cur_user = $plugin->app->session->read('logged_in_user');
@@ -528,6 +564,8 @@ sub update_user {
 
 sub user_has_role {
     my $plugin = shift;
+    croak "Cannot call user_has_role since roles are disabled by disable_roles setting"
+      if $plugin->disable_roles;
 
     my ( $username, $want_role );
     if ( @_ == 2 ) {
@@ -557,7 +595,7 @@ sub user_password {
     my @realms_to_check =
       $params{realm}
       ? ( $params{realm} )
-      : ( keys %{ $plugin->realms } );
+      : @{ $plugin->realm_names };
 
     # Expect either a code, username or nothing (for logged-in user)
     if ( exists $params{code} ) {
@@ -634,6 +672,8 @@ sub user_password {
 
 sub user_roles {
     my ( $plugin, $username, $realm ) = @_;
+    croak "Cannot call user_roles since roles are disabled by disable_roles setting"
+      if $plugin->disable_roles;
 
     $username = $plugin->app->session->read('logged_in_user')
       unless defined $username;
@@ -919,7 +959,7 @@ sub _send_email {
 # FIXME: this is not used anywhere - what is it for?
 sub _try_realms {
     my ( $plugin, $method, @args );
-    for my $realm ( keys %{ $plugin->realms } ) {
+    for my $realm ( @{ $plugin->realm_names } ) {
         my $provider = $plugin->auth_provider($realm);
         if ( !$provider->can($method) ) {
             croak "Provider $provider does not provide a $method method!";
@@ -1159,6 +1199,14 @@ Authenticates users stored in a database table using L<Dancer2::Plugin::DBIC>
 
 Authenticates users stored in a database table
 
+=item L<Dancer2::Plugin::Auth::Extensible::Provider::IMAP>
+
+Authenticates users via in an IMAP server.
+
+=item L<Dancer2::Plugin::Auth::Extensible::Provider::LDAP>
+
+Authenticates users stored in an LDAP directory.
+
 =back
 
 Need to write your own?  Just create a new provider class which consumes
@@ -1188,6 +1236,9 @@ the user is not logged in, they will be redirected to the login page URL.  If
 they are logged in, but do not have the required role, they will be redirected
 to the access denied URL.
 
+If C<disable_roles> configuration option is set to a true value then using
+L</require_role> will cause the application to croak on load.
+
 =item require_any_roles - require the user to have one of a list of roles
 
     get '/drink' => require_any_role [qw(BeerDrinker VodaDrinker)] => sub {
@@ -1199,6 +1250,9 @@ roles listed.  If the user is not logged in, they will be redirected to the
 login page URL.  If they are logged in, but do not have any of the specified
 roles, they will be redirected to the access denied URL.
 
+If C<disable_roles> configuration option is set to a true value then using
+L</require_any_roles> will cause the application to croak on load.
+
 =item require_all_roles - require the user to have all roles listed
 
     get '/foo' => require_all_roles [qw(Foo Bar)] => sub { ... };
@@ -1208,6 +1262,9 @@ If the user is not logged in, they will be redirected to the login page URL.  If
 they are logged in but do not have all of the specified roles, they will be
 redirected to the access denied URL.
 
+If C<disable_roles> configuration option is set to a true value then using
+L</require_all_roles> will cause the application to croak on load.
+
 =back
 
 =head2 Replacing the Default C< /login > and C< /login/denied > Routes
@@ -1337,6 +1394,9 @@ You can also provide the username to check;
 
     if (user_has_role($user, $role)) { .... }
 
+If C<disable_roles> configuration option is set to a true value then using
+L</user_has_role> will cause the application to croak at runtime.
+
 =item user_roles
 
 Returns a list of the roles of a user.
@@ -1346,6 +1406,9 @@ alternatively, you may supply a username to check.
 
 Returns a list or arrayref depending on context.
 
+If C<disable_roles> configuration option is set to a true value then using
+L</user_roles> will cause the application to croak at runtime.
+
 =item authenticate_user
 
 Usually you'll want to let the built-in login handling code deal with
@@ -1690,6 +1753,9 @@ In your application's configuation file:
     plugins:
         Auth::Extensible:
             # Set to 1 if you want to disable the use of roles (0 is default)
+            # If roles are disabled then any use of role-based route decorators
+            # will cause app to croak on load. Use of 'user_roles' and
+            # 'user_has_role' will croak at runtime.
             disable_roles: 0
             # After /login: If no return_url is given: land here ('/' is default)
             user_home_page: '/user'
@@ -1723,8 +1789,12 @@ In your application's configuation file:
             # you wish to use)
             realms:
                 realm_one:
+                    priority: 3 # Defaults to 0. Realms are checked in descending order
                     provider: Database
                         db_connection_name: 'foo'
+                realm_two:
+                    priority: 0 # Will be checked after realm_one
+                    provider: Config
 
 B<Please note> that you B<must> have a session provider configured.  The 
 authentication framework requires sessions in order to track information about 
@@ -1765,7 +1835,7 @@ Dancer2 port of Dancer::Plugin::Auth::Extensible by:
 
 Stefan Hornburg (Racke), C<< <racke at linuxia.de> >>
 
-Conversion to Dancer2's new plugin system in 2016 by:
+Conversion to Dancer2's new plugin system plus much cleanup & reorg:
 
 Peter Mottram (SysPete), C<< <peter at sysnix.com> >>
 
