@@ -4,7 +4,7 @@ use Crypt::SaltedHash;
 use Moo::Role;
 requires qw(authenticate_user get_user_details get_user_roles);
 
-our $VERSION = '0.613';
+our $VERSION = '0.614';
 
 =head1 NAME
 
@@ -30,6 +30,7 @@ Required.
 has plugin => (
     is       => 'ro',
     required => 1,
+    weaken   => 1,
 );
 
 =head2 disable_roles
@@ -48,13 +49,13 @@ has disable_roles => (
 
 The encryption_algorithm used by L</encrypt_password>.
 
-Defaults to 'SHA-1';
+Defaults to 'SHA-512';
 
 =cut
 
 has encryption_algorithm => (
     is      => 'ro',
-    default => 'SHA-1',
+    default => 'SHA-512',
 );
 
 =head1 METHODS
@@ -129,6 +130,10 @@ provider.
 
 =over
 
+=item * create_user
+
+=item * get_user_by_code
+
 =item * set_user_details
 
 =item * set_user_password
